@@ -36,6 +36,7 @@
 #include "php_scandir.h"
 #include "SAPI.h"
 #include "TSRM.h"
+#include "php_main.h"
 #include "ext/standard/md5.h"
 #include "ext/standard/php_var.h"
 #include "ext/standard/php_smart_str.h"
@@ -179,8 +180,6 @@ PHP_APCU_API void apc_cache_remove_slot(apc_cache_t* cache, apc_cache_slot_t** s
 PHP_APCU_API void apc_cache_gc(apc_cache_t* cache TSRMLS_DC)
 {
     apc_cache_slot_t** slot;
-	
-    time_t now;
 
     /* This function scans the list of removed cache entries and deletes any
      * entry whose reference count is zero  or that has been on the gc 
@@ -201,12 +200,11 @@ PHP_APCU_API void apc_cache_gc(apc_cache_t* cache TSRMLS_DC)
     {
 		slot = &cache->header->gc;
 
-		now = time(0);
-
 		while (*slot != NULL) {
-			int gc_sec = cache->gc_ttl ? (now - (*slot)->dtime) : 0;
+			time_t now = time(0);
+			time_t gc_sec = cache->gc_ttl ? (now - (*slot)->dtime) : 0;
 
-			if (!(*slot)->value->ref_count || gc_sec > cache->gc_ttl) {
+			if (!(*slot)->value->ref_count || gc_sec > (time_t)cache->gc_ttl) {
                 apc_cache_slot_t* dead = *slot;
 
 				/* good ol' whining */
@@ -265,9 +263,8 @@ PHP_APCU_API int APC_SERIALIZER_NAME(eval) (APC_SERIALIZER_ARGS)
                 "%s/apcu.%s",
                 APCG(writable), key->str
            ))) {
-                char *pathed;
                 php_stream *handle = php_stream_open_wrapper(
-                    path, "w+b", IGNORE_PATH, &pathed TSRMLS_CC);
+                    path, "w+b", IGNORE_PATH, NULL);
                 
                 if (handle) {
                     (*buf) = emalloc((*buf_len)+1);
@@ -299,7 +296,7 @@ PHP_APCU_API int APC_UNSERIALIZER_NAME(eval) (APC_UNSERIALIZER_ARGS)
 {
     zend_file_handle zhandle;
     
-    if (php_stream_open_for_zend_ex(buf, &zhandle, USE_PATH|STREAM_OPEN_FOR_INCLUDE TSRMLS_CC) == SUCCESS) {
+    if (php_stream_open_for_zend_ex((const char *)buf, &zhandle, USE_PATH|STREAM_OPEN_FOR_INCLUDE TSRMLS_CC) == SUCCESS) {
        zend_op_array *op_array = zend_compile_file(&zhandle, ZEND_INCLUDE TSRMLS_CC);
        zend_op_array *active_op_array = EG(active_op_array);
        zval **return_value_ptr_ptr = EG(return_value_ptr_ptr);
@@ -479,12 +476,14 @@ static zval* data_unserialize(const char *filename TSRMLS_DC)
     tmp = contents = malloc(len);
 
     if(!contents) {
-       return NULL;
+        fclose(fp);
+        return NULL;
     }
 
     if(fread(contents, 1, len, fp) < 1) {	
-      free(contents);
-      return NULL;
+        fclose(fp);
+        free(contents);
+        return NULL;
     }
 
     MAKE_STD_ZVAL(retval);
@@ -493,6 +492,7 @@ static zval* data_unserialize(const char *filename TSRMLS_DC)
     
     /* I wish I could use json */
     if(!php_var_unserialize(&retval, (const unsigned char**)&tmp, (const unsigned char*)(contents+len), &var_hash TSRMLS_CC)) {
+        fclose(fp);
         zval_ptr_dtor(&retval);
         return NULL;
     }
@@ -579,6 +579,10 @@ PHP_APCU_API void apc_cache_release(apc_cache_t* cache, apc_cache_entry_t* entry
 /* {{{ apc_cache_destroy */
 PHP_APCU_API void apc_cache_destroy(apc_cache_t* cache TSRMLS_DC)
 {
+	if (!cache) {
+		return;
+	}
+
 	/* destroy lock */
 	DESTROY_LOCK(&cache->header->lock);
 
@@ -747,7 +751,7 @@ PHP_APCU_API zend_bool apc_cache_make_context(apc_cache_t* cache,
 				context,
                 cache->serializer,
 				(apc_malloc_t) cache->sma->smalloc,
-                cache->sma->free,
+                cache->sma->sfree,
                 cache->sma->protect,
                 cache->sma->unprotect,
 				pool_type, copy_type, force_update TSRMLS_CC
@@ -875,7 +879,7 @@ PHP_APCU_API zend_bool apc_cache_insert(apc_cache_t* cache,
 		     * access ttl on it and removing entries that haven't been accessed for ttl seconds and secondly
 		     * we see if the entry has a hard ttl on it and remove it if it has been around longer than its ttl
 		     */
-		    if((cache->ttl && (*slot)->atime < (t - cache->ttl)) || 
+		    if((cache->ttl && (time_t)(*slot)->atime < (t - (time_t)cache->ttl)) || 
 		       ((*slot)->value->ttl && (time_t) ((*slot)->ctime + (*slot)->value->ttl) < t)) {
                 apc_cache_remove_slot(cache, slot TSRMLS_CC);
 		        continue;
@@ -1621,7 +1625,7 @@ PHP_APCU_API zval* apc_cache_info(apc_cache_t* cache, zend_bool limited TSRMLS_D
     zval *gc = NULL;
     zval *slots = NULL;
     apc_cache_slot_t* p;
-    int i, j;
+    zend_ulong i, j;
 
     if(!cache) {
 		return NULL;
@@ -1698,6 +1702,52 @@ PHP_APCU_API zval* apc_cache_info(apc_cache_t* cache, zend_bool limited TSRMLS_D
 }
 /* }}} */
 
+/*
+ fetches information about the key provided
+*/
+PHP_APCU_API zval* apc_cache_stat(apc_cache_t* cache,
+                                  char *strkey,
+                                  zend_uint keylen TSRMLS_DC) {
+    zval *stat;
+    apc_cache_slot_t** slot;
+	zend_ulong h, s;
+    
+	/* calculate hash and slot */
+	apc_cache_hash_slot(cache, strkey, keylen, &h, &s);
+	
+	/* allocate stat buffer */
+	ALLOC_INIT_ZVAL(stat);
+
+    /* read lock header */
+	APC_RLOCK(cache->header);
+
+	/* find head */
+	slot = &cache->slots[s];
+
+	while (*slot) {
+		/* check for a matching key by has and identifier */
+	    if ((h == (*slot)->key.h) && !memcmp((*slot)->key.str, strkey, keylen)) {
+            array_init(stat);
+            
+            add_assoc_long(stat, "hits",  (*slot)->nhits);
+            add_assoc_long(stat, "atime", (*slot)->atime);
+            add_assoc_long(stat, "ctime", (*slot)->ctime);
+            add_assoc_long(stat, "dtime", (*slot)->dtime);
+	        add_assoc_long(stat, "ttl",   (*slot)->value->ttl);
+	        add_assoc_long(stat, "refs",  (*slot)->value->ref_count);
+	        
+	        break;
+	    }
+
+		/* next */
+	    slot = &(*slot)->next;		
+	}
+    
+    APC_RUNLOCK(cache->header);
+    
+    return stat;
+}
+
 /* {{{ apc_cache_busy */
 PHP_APCU_API zend_bool apc_cache_busy(apc_cache_t* cache TSRMLS_DC)
 {	
