@@ -28,6 +28,10 @@
 
 /* $Id: php_apc.c 328290 2012-11-09 03:30:09Z laruence $ */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "apc_cache.h"
 #include "apc_iterator.h"
 #include "apc_sma.h"
@@ -57,6 +61,7 @@
 PHP_FUNCTION(apcu_cache_info);
 PHP_FUNCTION(apcu_clear_cache);
 PHP_FUNCTION(apcu_sma_info);
+PHP_FUNCTION(apcu_key_info);
 PHP_FUNCTION(apcu_store);
 PHP_FUNCTION(apcu_fetch);
 PHP_FUNCTION(apcu_delete);
@@ -188,7 +193,7 @@ STD_PHP_INI_ENTRY("apc.rfc1867_ttl", "3600", PHP_INI_SYSTEM, OnUpdateLong, rfc18
 STD_PHP_INI_ENTRY("apc.preload_path", (char*)NULL,              PHP_INI_SYSTEM, OnUpdateString,       preload_path,  zend_apcu_globals, apcu_globals)
 STD_PHP_INI_BOOLEAN("apc.coredump_unmap", "0", PHP_INI_SYSTEM, OnUpdateBool, coredump_unmap, zend_apcu_globals, apcu_globals)
 STD_PHP_INI_BOOLEAN("apc.use_request_time", "1", PHP_INI_ALL, OnUpdateBool, use_request_time,  zend_apcu_globals, apcu_globals)
-STD_PHP_INI_ENTRY("apc.serializer", "default", PHP_INI_SYSTEM, OnUpdateStringUnempty, serializer_name, zend_apcu_globals, apcu_globals)
+STD_PHP_INI_ENTRY("apc.serializer", "php", PHP_INI_SYSTEM, OnUpdateStringUnempty, serializer_name, zend_apcu_globals, apcu_globals)
 STD_PHP_INI_ENTRY("apc.writable", "/tmp", PHP_INI_SYSTEM, OnUpdateStringUnempty, writable, zend_apcu_globals, apcu_globals)
 PHP_INI_END()
 
@@ -277,14 +282,33 @@ static PHP_MINIT_FUNCTION(apcu)
 			apc_sma.init(APCG(shm_segments), APCG(shm_size), NULL TSRMLS_CC);
 #endif
 
+/* XXX pack this into macros when there are more hooks to handle */
+#if defined(PHP_WIN32) && defined(_WIN64)
+			do {
+				 char buf[65];
+
+				 if (!_i64toa_s((__int64)_apc_register_serializer, buf, 65, 10)) {
+					REGISTER_STRING_CONSTANT(APC_SERIALIZER_CONSTANT, buf, CONST_PERSISTENT | CONST_CS);
+				 } else {
+					/* subsequent apc_register_serializer() calls will be void */
+					php_error_docref(NULL TSRMLS_CC, E_WARNING, "Serializer hook init failed");
+				 }
+			} while (0);
+#else
+			REGISTER_LONG_CONSTANT(APC_SERIALIZER_CONSTANT, (long)&_apc_register_serializer, CONST_PERSISTENT | CONST_CS);
+#endif
+
 			/* register default serializer */
-			apc_register_serializer(
+			_apc_register_serializer(
 				"php", APC_SERIALIZER_NAME(php), APC_UNSERIALIZER_NAME(php), NULL TSRMLS_CC);
 				
 			/* register eval serializer */
-			apc_register_serializer(
+			_apc_register_serializer(
 				"eval", APC_SERIALIZER_NAME(eval), APC_UNSERIALIZER_NAME(eval), NULL TSRMLS_CC);
-			
+
+			/* test out the constant function pointer */
+			assert(apc_get_serializers(TSRMLS_C)->name != NULL);
+
 			/* create user cache */
 			apc_user_cache = apc_cache_create(
 				&apc_sma,
@@ -313,10 +337,11 @@ static PHP_MINIT_FUNCTION(apcu)
             apc_iterator_init(module_number TSRMLS_CC);
         }
 
-        zend_register_long_constant("APC_BIN_VERIFY_MD5", sizeof("APC_BIN_VERIFY_MD5"), APC_BIN_VERIFY_MD5, (CONST_CS | CONST_PERSISTENT), module_number TSRMLS_CC);
-        zend_register_long_constant("APC_BIN_VERIFY_CRC32", sizeof("APC_BIN_VERIFY_CRC32"), APC_BIN_VERIFY_CRC32, (CONST_CS | CONST_PERSISTENT), module_number TSRMLS_CC);
+        REGISTER_LONG_CONSTANT("APC_BIN_VERIFY_MD5", APC_BIN_VERIFY_MD5, CONST_CS | CONST_PERSISTENT);
+        REGISTER_LONG_CONSTANT("APC_BIN_VERIFY_CRC32", APC_BIN_VERIFY_CRC32, CONST_CS | CONST_PERSISTENT);
     }
 
+#ifndef REGISTER_BOOL_CONSTANT
     {
         zend_constant apc_bc;
         Z_TYPE(apc_bc.value) = IS_BOOL;
@@ -331,6 +356,14 @@ static PHP_MINIT_FUNCTION(apcu)
         apc_bc.module_number = module_number;
         zend_register_constant(&apc_bc TSRMLS_CC);
     }
+#else
+#if defined(APC_FULL_BC) && APC_FULL_BC
+    REGISTER_BOOL_CONSTANT("APCU_APC_FULL_BC", 1, CONST_CS | CONST_PERSISTENT);
+#else
+    REGISTER_BOOL_CONSTANT("APCU_APC_FULL_BC", 0, CONST_CS | CONST_PERSISTENT);
+#endif
+#endif
+
 #ifdef APC_FULL_BC
 	apc_init(INIT_FUNC_ARGS_PASSTHRU);
 #endif
@@ -387,10 +420,55 @@ static PHP_RINIT_FUNCTION(apcu)
 }
 /* }}} */
 
+#ifdef APC_FULL_BC
+/* {{{ proto void apc_clear_cache([string cache]) */
+PHP_FUNCTION(apcu_clear_cache)
+{
+    char *ignored;
+    int ignlen = 0;
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &ignored, &ignlen) == FAILURE) {
+        return;
+    }
+
+    if (0 == ignlen || APC_CACHE_IS_USER(ignored, ignlen)) {
+        apc_cache_clear(apc_user_cache TSRMLS_CC);
+    }
+
+    RETURN_TRUE;
+}
+/* }}} */
+
+/* {{{ proto array apc_cache_info(string cache_type, [bool limited]) */
+PHP_FUNCTION(apcu_cache_info)
+{
+    zval* info;
+    zend_bool limited = 0;
+    char *ct;
+    ulong ctlen;
+    
+    if (ZEND_NUM_ARGS()) {
+        if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|b", &ct, &ctlen, &limited) == FAILURE) {
+            return;
+        }
+    }
+
+    info = apc_cache_info(apc_user_cache, limited TSRMLS_CC);
+
+    if (!info) {
+        php_error_docref(NULL TSRMLS_CC, E_WARNING, "No APC info available.  Perhaps APC is not enabled? Check apc.enabled in your ini file");
+        RETURN_FALSE;
+    }
+
+    RETURN_ZVAL(info, 0, 1);
+
+}
+/* }}} */
+#else
 /* {{{ proto void apc_clear_cache() */
 PHP_FUNCTION(apcu_clear_cache)
 {
-    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "") == FAILURE) {
+    if (zend_parse_parameters_none() == FAILURE) {
         return;
     }
 
@@ -421,6 +499,23 @@ PHP_FUNCTION(apcu_cache_info)
 
 }
 /* }}} */
+#endif
+
+PHP_FUNCTION(apcu_key_info)
+{
+    zval *stat;
+    char *strkey;
+    zend_uint keylen;
+    
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &strkey, &keylen) == FAILURE) {
+        return;
+    }
+    
+    stat = apc_cache_stat(
+        apc_user_cache, strkey, keylen+1 TSRMLS_CC);
+    
+    RETURN_ZVAL(stat, 0, 1);
+}
 
 /* {{{ proto array apc_sma_info([bool limited]) */
 PHP_FUNCTION(apcu_sma_info)
@@ -781,7 +876,8 @@ PHP_FUNCTION(apcu_fetch) {
 				    if (Z_TYPE_PP(hentry) == IS_STRING) {
 
 				        /* perform find using this index as key */
-						if ((entry = apc_cache_find(apc_user_cache, Z_STRVAL_PP(hentry), (Z_STRLEN_PP(hentry) + 1), t TSRMLS_CC))) {
+						if ((entry = apc_cache_find(apc_user_cache, Z_STRVAL_PP(hentry), (Z_STRLEN_PP(hentry) + 1), t 
+TSRMLS_CC))) {
 							zval *result_entry;
 
 						    /* deep-copy returned shm zval to emalloc'ed return_value */
@@ -793,7 +889,8 @@ PHP_FUNCTION(apcu_fetch) {
 								apc_user_cache, entry TSRMLS_CC);
 							/* add the emalloced value to return array */
 						    zend_hash_add(
-								Z_ARRVAL_P(result), Z_STRVAL_PP(hentry), Z_STRLEN_PP(hentry) +1, &result_entry, sizeof(zval*), NULL);
+								Z_ARRVAL_P(result), Z_STRVAL_PP(hentry), Z_STRLEN_PP(hentry) +1, &result_entry, 
+sizeof(zval*), NULL);
 						}
 				    } else {
 
@@ -1092,7 +1189,7 @@ PHP_FUNCTION(apcu_bin_dumpfile) {
 PHP_FUNCTION(apc_bin_dumpfile) {
 
     zval *z_files = NULL, *z_user_vars = NULL;
-    HashTable *h_files, *h_user_vars;
+    HashTable *h_user_vars;
     char *filename = NULL;
     int filename_len;
     long flags=0;
@@ -1108,7 +1205,8 @@ PHP_FUNCTION(apc_bin_dumpfile) {
     }
 
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a!a!s|lr!", &z_files, &z_user_vars, &filename, &filename_len, &flags, &zcontext) == FAILURE) {
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a!a!s|lr!", &z_files, &z_user_vars, &filename, &filename_len, &flags, &zcontext) == 
+FAILURE) {
         return;
     }
 
@@ -1117,7 +1215,6 @@ PHP_FUNCTION(apc_bin_dumpfile) {
         RETURN_FALSE;
     }
 
-    h_files = z_files ? Z_ARRVAL_P(z_files) : NULL;
     h_user_vars = z_user_vars ? Z_ARRVAL_P(z_user_vars) : NULL;
     bd = apc_bin_dump(apc_user_cache, h_user_vars TSRMLS_CC);
     if(!bd) {
@@ -1181,7 +1278,8 @@ PHP_FUNCTION(apcu_bin_load) {
     }
 
     if (!data_len || data_len != ((apc_bd_t*)data)->size -1) {
-        apc_error("apc_bin_load string argument does not appear to be a valid APC binary dump due to size (%d vs expected %d)." TSRMLS_CC, data_len, ((apc_bd_t*)data)->size -1);
+        apc_error("apc_bin_load string argument does not appear to be a valid APC binary dump due to size (%d vs expected %d)." TSRMLS_CC, data_len, 
+((apc_bd_t*)data)->size -1);
         RETURN_FALSE;
     }
 
@@ -1235,7 +1333,8 @@ PHP_FUNCTION(apcu_bin_loadfile) {
         apc_warning("Error reading file passed to apc_bin_loadfile: %s." TSRMLS_CC, filename);
         RETURN_FALSE;
     } else if (len != ((apc_bd_t*)data)->size) {
-        apc_warning("file passed to apc_bin_loadfile does not appear to be valid due to size (%d vs expected %d)." TSRMLS_CC, len, ((apc_bd_t*)data)->size -1);
+        apc_warning("file passed to apc_bin_loadfile does not appear to be valid due to size (%d vs expected %d)." TSRMLS_CC, len, 
+((apc_bd_t*)data)->size -1);
         RETURN_FALSE;
     }
     php_stream_close(stream);
@@ -1261,15 +1360,31 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_apcu_store, 0, 0, 2)
     ZEND_ARG_INFO(0, ttl)
 ZEND_END_ARG_INFO()
 
+#ifdef APC_FULL_BC
 PHP_APC_ARGINFO
+/* this will generate different reflection but retains functional compatibility */
 ZEND_BEGIN_ARG_INFO_EX(arginfo_apcu_cache_info, 0, 0, 0)
     ZEND_ARG_INFO(0, type)
     ZEND_ARG_INFO(0, limited)
 ZEND_END_ARG_INFO()
+PHP_APC_ARGINFO
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apcu_clear_cache, 0, 0, 0)
+    ZEND_ARG_INFO(0, cache)
+ZEND_END_ARG_INFO()
+#else
+PHP_APC_ARGINFO
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apcu_cache_info, 0, 0, 0)
+    ZEND_ARG_INFO(0, limited)
+ZEND_END_ARG_INFO()
 
 PHP_APC_ARGINFO
 ZEND_BEGIN_ARG_INFO_EX(arginfo_apcu_clear_cache, 0, 0, 0)
-    ZEND_ARG_INFO(0, info)
+ZEND_END_ARG_INFO()
+#endif
+
+PHP_APC_ARGINFO
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apcu_key_info, 0, 0, 1)
+    ZEND_ARG_INFO(0, key)
 ZEND_END_ARG_INFO()
 
 PHP_APC_ARGINFO
@@ -1288,6 +1403,7 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_apcu_fetch, 0, 0, 1)
     ZEND_ARG_INFO(1, success)
 ZEND_END_ARG_INFO()
 
+
 PHP_APC_ARGINFO
 ZEND_BEGIN_ARG_INFO_EX(arginfo_apcu_inc, 0, 0, 1)
     ZEND_ARG_INFO(0, key)
@@ -1357,6 +1473,7 @@ zend_function_entry apcu_functions[] = {
     PHP_FE(apcu_cache_info,         arginfo_apcu_cache_info)
     PHP_FE(apcu_clear_cache,        arginfo_apcu_clear_cache)
     PHP_FE(apcu_sma_info,           arginfo_apcu_sma_info)
+    PHP_FE(apcu_key_info,           arginfo_apcu_key_info)
     PHP_FE(apcu_store,              arginfo_apcu_store)
     PHP_FE(apcu_fetch,              arginfo_apcu_fetch)
     PHP_FE(apcu_delete,             arginfo_apcu_delete)
